/*
 * gaim
 *
 * Copyright (C) 1998-1999, Mark Spencer <markster@marko.net>
 *
 * osd.c: On-screen display plugin for Gaim.
 * Copyright (C) 2001 Chris Sperandio <csperandio@users.sourceforge.net>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */
/* (cvs $Id: gaimosd.c,v 1.20 2004/09/20 06:45:57 deryni9 Exp $) */

#include "gaimosd.h"

/*
	GAIM IPC ideas:

	- display OSD message command
	- get build options
*/

#if DYNAMIC_XOSD
static GModule *Xosd_handle;
#endif /* DYNAMIC_XOSD */

/* GLOBAL VARIABLES */
osd_display_t *Osd_current = NULL;
gint Osd_lines = 0;

static gboolean
plugin_load(GaimPlugin *plugin)
{
#if DYNAMIC_XOSD
	gboolean dl_fail = TRUE;

	if ((Xosd_handle = g_module_open(XOSD_LIBRARY, 0))) {
		dl_fail = FALSE;
		dl_fail |= !g_module_symbol (Xosd_handle, "xosd_create",
		                             (gpointer *) &dl_xosd_create);
		dl_fail |= !g_module_symbol (Xosd_handle, "xosd_destroy",
		                             (gpointer *) &dl_xosd_destroy);
		dl_fail |= !g_module_symbol (Xosd_handle, "xosd_display",
		                             (gpointer *) &dl_xosd_display);
		dl_fail |= !g_module_symbol (Xosd_handle, "xosd_is_onscreen",
		                             (gpointer *) &dl_xosd_is_onscreen);
		dl_fail |= !g_module_symbol (Xosd_handle, "xosd_wait_until_no_display",
		                             (gpointer *) &dl_xosd_wait_until_no_display);
		dl_fail |= !g_module_symbol (Xosd_handle, "xosd_hide",
		                             (gpointer *) &dl_xosd_hide);
		dl_fail |= !g_module_symbol (Xosd_handle, "xosd_show",
		                             (gpointer *) &dl_xosd_show);
		dl_fail |= !g_module_symbol (Xosd_handle, "xosd_set_pos",
		                             (gpointer *) &dl_xosd_set_pos);
		dl_fail |= !g_module_symbol (Xosd_handle, "xosd_set_align",
		                             (gpointer *) &dl_xosd_set_align);
		dl_fail |= !g_module_symbol (Xosd_handle, "xosd_set_vertical_offset",
		                             (gpointer *) &dl_xosd_set_vertical_offset);
		dl_fail |= !g_module_symbol (Xosd_handle, "xosd_set_horizontal_offset",
		                             (gpointer *) &dl_xosd_set_horizontal_offset);
		dl_fail |= !g_module_symbol (Xosd_handle, "xosd_set_shadow_offset",
		                             (gpointer *) &dl_xosd_set_shadow_offset);
		dl_fail |= !g_module_symbol (Xosd_handle, "xosd_set_timeout",
		                             (gpointer *) &dl_xosd_set_timeout);
		dl_fail |= !g_module_symbol (Xosd_handle, "xosd_set_font",
		                             (gpointer *) &dl_xosd_set_font);
		dl_fail |= !g_module_symbol (Xosd_handle, "xosd_set_colour",
		                             (gpointer *) &dl_xosd_set_colour);
		dl_fail |= !g_module_symbol (Xosd_handle, "xosd_get_colour",
		                             (gpointer *) &dl_xosd_get_colour);
		dl_fail |= !g_module_symbol (Xosd_handle, "xosd_scroll",
		                             (gpointer *) &dl_xosd_scroll);
		dl_fail |= !g_module_symbol (Xosd_handle, "xosd_get_number_lines",
		                             (gpointer *) &dl_xosd_get_number_lines);
	}
	if (dl_fail) {
		gaim_notify_error(plugin, "Gaim OSD Error",
			                "Unable to load XOSD!", g_module_error());
		if (Xosd_handle)
			g_module_close(Xosd_handle);

		return FALSE;
	}
#endif /* DYNAMIC_XOSD */

	if (!osd_display_init()) {
		gaim_notify_error(plugin, "Gaim OSD Error",
			                "Unable to load XOSD!",
			                "Could not create XOSD handle");
#if DYNAMIC_XOSD
		g_module_close(Xosd_handle);
#endif
		return FALSE;
	}

#if BUILD_OSD_CONVS
	osd_conv_rebuild();
#endif
	osd_hooks_connect(plugin);

	return TRUE;
}

static gboolean
plugin_unload(GaimPlugin *plugin)
{
	osd_display_uninit();
#if BUILD_OSD_CONVS
	osd_conv_free_all();
#endif

#if DYNAMIC_XOSD
	g_module_close(Xosd_handle);
#endif
	return TRUE;
}


static GaimPluginInfo info =
{
	GAIM_PLUGIN_MAGIC,
	GAIM_MAJOR_VERSION,
	GAIM_MINOR_VERSION,
	GAIM_PLUGIN_STANDARD,                             /**< type           */
	NULL,                                             /**< ui_requirement */
	GAIM_PLUGIN_FLAG_INVISIBLE,                       /**< flags          */
	NULL,                                             /**< dependencies   */
	GAIM_PRIORITY_DEFAULT,                            /**< priority       */

	GAIMOSD_PLUGIN_ID,                                /**< id             */
	N_("Gaim OSD core"),                              /**< name           */
	GAIMOSD_VERSION,                                  /**< version        */
	                                                  /**< summary        */
	N_("On-screen display messages."),
	                                                  /**< description    */
	N_("Adds on-screen notification messages.  The effect is "
	   "similar to text overlay on a television for menus, volume, and "
	   "contrast control."),
	                                                  /**< author         */
	"\n\tPaul Miller <pmiller9@users.sourceforge.net>\n"
	"\tChris Sperandio <csperandio@users.sourceforge.net>\n",
	GAIMOSD_WEBSITE,                                  /**< homepage       */

	plugin_load,                                      /**< load           */
	plugin_unload,                                    /**< unload         */
	NULL,                                             /**< destroy        */

	NULL,                                             /**< ui_info        */
	NULL                                              /**< extra_info     */
};

static void
init_plugin(GaimPlugin *plugin)
{
	gchar *tmp, *node;
	gint i;

	gaim_prefs_add_none(GAIMOSD_PREFS);
	gaim_prefs_add_none(GAIMOSD_PREFS "/conv");
	gaim_prefs_add_none(GAIMOSD_PREFS "/chat");

	gaim_prefs_add_int(GAIMOSD_PREFS "/osd_lines", DEF_OSD_LINES);
	gaim_prefs_add_int(GAIMOSD_PREFS "/msg_length", DEF_MSG_LENGTH);
	gaim_prefs_add_int(GAIMOSD_PREFS "/signon_priority", DEF_SIGNON_PRIORITY);
	gaim_prefs_add_int(GAIMOSD_PREFS "/signon_timeout", DEF_SIGNON_TIMEOUT);
	gaim_prefs_add_bool(GAIMOSD_PREFS "/conv/display_def", DEF_CONV_DISPLAY_DEF);
	gaim_prefs_add_bool(GAIMOSD_PREFS "/conv/reply_hide", DEF_CONV_REPLY_HIDE);
	gaim_prefs_add_bool(GAIMOSD_PREFS "/conv/toolbar", DEF_CONV_TOOLBAR);
	gaim_prefs_add_bool(GAIMOSD_PREFS "/chat/display_def", DEF_CHAT_DISPLAY_DEF);
	gaim_prefs_add_bool(GAIMOSD_PREFS "/chat/me_override", DEF_CHAT_ME_OVERRIDE);
	gaim_prefs_add_bool(GAIMOSD_PREFS "/chat/reply_hide", DEF_CHAT_REPLY_HIDE);
	gaim_prefs_add_bool(GAIMOSD_PREFS "/chat/toolbar", DEF_CHAT_TOOLBAR);

	gaim_prefs_add_none(GAIMOSD_PREFS "/xosd");
	gaim_prefs_add_string(GAIMOSD_PREFS "/xosd/font", DEF_FONT);
	gaim_prefs_add_int(GAIMOSD_PREFS "/xosd/h_offset", DEF_H_OFFSET);
	gaim_prefs_add_int(GAIMOSD_PREFS "/xosd/h_align", DEF_H_ALIGN);
	gaim_prefs_add_int(GAIMOSD_PREFS "/xosd/v_offset", DEF_V_OFFSET);
	gaim_prefs_add_int(GAIMOSD_PREFS "/xosd/v_align", DEF_V_ALIGN);
	gaim_prefs_add_int(GAIMOSD_PREFS "/xosd/shadow_offset", DEF_SHADOW_OFFSET);

	gaim_prefs_add_none(GAIMOSD_PREFS "/event");
	for (i = 0; i < N_EVENTS; i++) {
		node = g_strconcat(GAIMOSD_PREFS "/event/", Osd_event[i].name, NULL);
		gaim_prefs_add_none(node);

		tmp = g_strconcat(node, "/priority", NULL);
		gaim_prefs_add_int(tmp, Osd_event[i].priority);
		g_free(tmp);

		tmp = g_strconcat(node, "/color", NULL);
		gaim_prefs_add_string(tmp, Osd_event[i].color);
		g_free(tmp);

		tmp = g_strconcat(node, "/flags", NULL);
		gaim_prefs_add_int(tmp, Osd_event[i].flags);
		g_free(tmp);

		tmp = g_strconcat(node, "/timeout", NULL);
		gaim_prefs_add_int(tmp, Osd_event[i].timeout);
		g_free(tmp);

		tmp = g_strconcat(node, "/text", NULL);
		gaim_prefs_add_string(tmp, Osd_event[i].text);
		g_free(tmp);
		g_free(node);
	}
}

GAIM_INIT_PLUGIN(gaimosd, init_plugin, info);
