/*
 * gaim
 *
 * Copyright (C) 1998-1999, Mark Spencer <markster@marko.net>
 *
 * osd.c: On-screen display plugin for Gaim.
 * Copyright (C) 2001 Chris Sperandio <csperandio@users.sourceforge.net>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */
/* (cvs $Id: gaimosd.h,v 1.24 2004/09/20 06:57:51 deryni9 Exp $) */

#ifndef __GAIMOSD_H__
#define __GAIMOSD_H__

/*
#include <stdio.h>
#include <fcntl.h>
*/
#include <xosd.h>
#include <glib.h>
#include <gmodule.h>

#if DYNAMIC_XOSD
#include <dlfcn.h>
#endif

/* gaim headers */
#include "internal.h"
#include "connection.h"
#include "conversation.h"
#include "core.h"
#include "debug.h"
#include "signals.h"
#include "util.h"
#include "notify.h"
#include "prefs.h"
#include "blist.h"
#include "prpl.h"
#include "version.h"

#ifdef HAVE_CONFIG_H
#include "gaimosd_config.h"
#endif

#if 0
/* enable this option for individually customizable buddies */
#define BUILD_OSD_BUDDIES FALSE
/* enable this option for OSD icons in conv/chat windows */
#define BUILD_OSD_CONVS   FALSE
/* dynamically load XOSD library */
#define DYNAMIC_XOSD      TRUE
#endif

#define GAIMOSD_PLUGIN_ID "gaimosd"
#define GAIMOSD_GTK_PLUGIN_ID "gaimosd-gtk"
#define GAIMOSD_QT_PLUGIN_ID "gaimosd-qt"
#define GAIMOSD_WEBSITE "http://gaimosd.sourceforge.net"

#define XOSD_WEBSITE "http://www.ignavus.net/software.html"

/* general */
#define DEF_FONT                  "-*-*-*-*-*-*-*-*-*-*-*-*-*-*"
#define DEF_H_OFFSET              0
#define DEF_H_ALIGN               XOSD_left
#define DEF_V_OFFSET              50
#define DEF_V_ALIGN               XOSD_bottom
#define DEF_SHADOW_OFFSET         2
#define DEF_MSG_LENGTH            80
#define DEF_OSD_LINES             2
#define DEF_SIGNON_PRIORITY       10
#define DEF_SIGNON_TIMEOUT        5000
#define DEF_TIMEOUT               5000
#define DEF_PRIORITY              50
#define DEF_BUDDY_PRIORITY        0
#define DEF_CONV_DISPLAY_DEF      TRUE
#define DEF_CONV_REPLY_HIDE       TRUE
#define DEF_CONV_TOOLBAR          TRUE
#define DEF_CHAT_DISPLAY_DEF      TRUE
#define DEF_CHAT_ME_OVERRIDE      TRUE
#define DEF_CHAT_REPLY_HIDE       TRUE
#define DEF_CHAT_TOOLBAR          TRUE
#define OSD_MAX_LINES             10

#define GAIMOSD_FLAGS_VOID        0x0000
#define GAIMOSD_FLAGS_VALID       0x0001
#define GAIMOSD_FLAGS_SHOW_HERE   0x0002
#define GAIMOSD_FLAGS_SHOW_AWAY   0x0004
#define GAIMOSD_FLAGS_SHOW_NEVER  0x0008

#define GAIMOSD_FLAGS_SHOW_ALWAYS  (GAIMOSD_FLAGS_SHOW_HERE | GAIMOSD_FLAGS_SHOW_AWAY)
#define GAIMOSD_FLAGS_SHOW_DEFAULT (GAIMOSD_FLAGS_VALID)

#define GAIMOSD_PREFS             "/plugins/gaimosd"

/* DYNAMIC_XOSD - libxosd.so */
#if DYNAMIC_XOSD
#define XOSD_LIBRARY "libxosd.so"

#define XOSD_CREATE (*dl_xosd_create)
#define XOSD_DESTROY (*dl_xosd_destroy)
#define XOSD_DISPLAY (*dl_xosd_display)
#define XOSD_IS_ONSCREEN (*dl_xosd_is_onscreen)
#define XOSD_WAIT_UNTIL_NO_DISPLAY (*dl_xosd_wait_until_no_display)
#define XOSD_HIDE (*dl_xosd_hide)
#define XOSD_SHOW (*dl_xosd_show)
#define XOSD_SET_POS (*dl_xosd_set_pos)
#define XOSD_SET_ALIGN (*dl_xosd_set_align)
#define XOSD_SET_VERTICAL_OFFSET (*dl_xosd_set_vertical_offset)
#define XOSD_SET_HORIZONTAL_OFFSET (*dl_xosd_set_horizontal_offset)
#define XOSD_SET_SHADOW_OFFSET (*dl_xosd_set_shadow_offset)
#define XOSD_SET_TIMEOUT (*dl_xosd_set_timeout)
#define XOSD_SET_FONT (*dl_xosd_set_font)
#define XOSD_SET_COLOUR (*dl_xosd_set_colour)
#define XOSD_GET_COLOUR (*dl_xosd_get_colour)
#define XOSD_SCROLL (*dl_xosd_scroll)
#define XOSD_GET_NUMBER_LINES (*dl_xosd_get_number_lines)

xosd* (*dl_xosd_create) (int);
int (*dl_xosd_destroy) (xosd*);
int (*dl_xosd_display) (xosd*, int, xosd_command, ...);
int (*dl_xosd_is_onscreen) (xosd*);
int (*dl_xosd_wait_until_no_display) (xosd*);
int (*dl_xosd_hide) (xosd*);
int (*dl_xosd_show) (xosd*);
int (*dl_xosd_set_pos) (xosd*, xosd_pos);
int (*dl_xosd_set_align) (xosd*, xosd_align);
int (*dl_xosd_set_vertical_offset) (xosd*, int);
int (*dl_xosd_set_horizontal_offset) (xosd*, int);
int (*dl_xosd_set_shadow_offset) (xosd*, int);
int (*dl_xosd_set_timeout) (xosd*, int);
int (*dl_xosd_set_font) (xosd*, char*);
int (*dl_xosd_set_colour) (xosd*, char*);
int (*dl_xosd_get_colour) (xosd*, int*, int*, int*);
int (*dl_xosd_scroll) (xosd*, int);
int (*dl_xosd_get_number_lines) (xosd*);

#else /* STATIC */

#define XOSD_CREATE xosd_create
#define XOSD_DESTROY xosd_destroy
#define XOSD_DISPLAY xosd_display
#define XOSD_IS_ONSCREEN xosd_is_onscreen
#define XOSD_WAIT_UNTIL_NO_DISPLAY xosd_wait_until_no_display
#define XOSD_HIDE xosd_hide
#define XOSD_SHOW xosd_show
#define XOSD_SET_POS xosd_set_pos
#define XOSD_SET_ALIGN xosd_set_align
#define XOSD_SET_VERTICAL_OFFSET xosd_set_vertical_offset
#define XOSD_SET_HORIZONTAL_OFFSET xosd_set_horizontal_offset
#define XOSD_SET_SHADOW_OFFSET xosd_set_shadow_offset
#define XOSD_SET_TIMEOUT xosd_set_timeout
#define XOSD_SET_FONT xosd_set_font
#define XOSD_SET_COLOUR xosd_set_colour
#define XOSD_GET_COLOUR xosd_get_colour
#define XOSD_SCROLL xosd_scroll
#define XOSD_GET_NUMBER_LINES xosd_get_number_lines

#endif /* DYNAMIC_XOSD */

enum osd_events {
	EVENT_BUDDY_SIGNED_ON,
	EVENT_BUDDY_SIGNED_OFF,
	EVENT_BUDDY_AWAY,
	EVENT_BUDDY_BACK,
	EVENT_BUDDY_IDLE,
	EVENT_BUDDY_UNIDLE,
	EVENT_CONVERSATION_CREATED,
	EVENT_BUDDY_TYPING,
	EVENT_RECEIVED_IM_MSG,
	EVENT_RECEIVED_CHAT_MSG,
	EVENT_RECEIVED_CHAT_MSG_ME,
	EVENT_CHAT_BUDDY_JOINED,
	EVENT_CHAT_BUDDY_LEFT,
	EVENT_CHAT_INVITED,
	EVENT_CHAT_JOINED,
	EVENT_ACCOUNT_WARNED,

	N_EVENTS,
};

typedef struct {
	GSList *slist;
	gint lines;
	gint priority;
	guint timeout;
	GaimConnection *gc;
	GaimConversation *c;
} osd_display_t;

typedef struct {
	GaimConversation *conv;
	gboolean display;
} osd_conv_t;

typedef struct {
	const gchar *name;
	const gchar *desc;
	const gchar *color;
	const gint timeout;
	const gint priority;
	const gint flags;
	const gchar *text;
} osd_event_t;

extern osd_display_t *Osd_current;
extern gint Osd_lines;

/* DISPLAY.C */
extern gboolean osd_display_cb(osd_display_t *disp);
void osd_display(GaimConnection *gc, GaimConversation *c,
                 gchar *who, gint event, gchar *m, gchar *o);
void osd_display_kill(void);
gboolean osd_display_init(void);
void osd_display_uninit(void);

/* CONV.C */
#if BUILD_OSD_CONVS
osd_conv_t *osd_conv_new(GaimConversation *conv);
void osd_conv_rebuild(void);
void osd_conv_free(gpointer data, gpointer p);
void osd_conv_free_all(void);
osd_conv_t *osd_conv_find(GaimConversation *conv);
void osd_conv_delete(GaimConversation *conv);
#endif /* BUILD_OSD_CONVS */

/* GAIMOSD.C */

/* HOOKS.C */
void osd_hooks_connect(GaimPlugin *plugin);




/* table tab width: 4 characters */

static const osd_event_t Osd_event[N_EVENTS] =
{
	{"buddy_signed_on",		"Buddy signed on",
							"#7bfc00",	5000,	40,
							GAIMOSD_FLAGS_SHOW_HERE,
							"%p Buddy Signon:\n%a"},

	{"buddy_signed_off",	"Buddy signed off",
							"#7bfc00",	5000,	40,
							GAIMOSD_FLAGS_SHOW_HERE,
							"%p Buddy Signoff:\n%a"},

	{"buddy_away",			"Buddy went away",
							"#1a95fc",	3000,	50,
							GAIMOSD_FLAGS_SHOW_DEFAULT,
							"Buddy Away:\n%a"},

	{"buddy_back",			"Buddy came back",
							"#1a95fc",	3000,	50,
							GAIMOSD_FLAGS_SHOW_DEFAULT,
							"Buddy Returned:\n%a"},

	{"buddy_idle",			"Buddy became idle",
							"#1a95fc",	3000,	60,
							GAIMOSD_FLAGS_SHOW_DEFAULT,
							"Buddy Idle:\n%a"},

	{"buddy_unidle",		"Buddy is no longer idle",
							"#1a95fc",	3000,	60,
							GAIMOSD_FLAGS_SHOW_DEFAULT,
							"Buddy Returned:\n%a"},

	{"conversation_created","New conversation created",
							"#ffff00",	3000,	50,
							GAIMOSD_FLAGS_SHOW_DEFAULT,
							"New Conversation:\n%a"},

	{"buddy_typing",		"Buddy is typing",
							"#ffff00",	3000,	60,
							GAIMOSD_FLAGS_SHOW_HERE,
							"%a typing..."},

	{"received_im",			"Message received",
							"#ffff00",	10000,	10,
							GAIMOSD_FLAGS_SHOW_HERE,
							"%a:  %m"},

	{"received_chat_msg",	"Chat message received",
							"#da60a5",	10000,	30,
							GAIMOSD_FLAGS_SHOW_DEFAULT,
							"(%o) %a:  %m"},

	{"received_chat_msg_me","Chat message received with screenname",
							"#ffff00",	10000,	10,
							GAIMOSD_FLAGS_SHOW_HERE,
							"(%o) %a:  %m"},

	{"chat_buddy_joined",	"Buddy joined chatroom",
							"#d2d2d2",	5000,	70,
							GAIMOSD_FLAGS_SHOW_DEFAULT,
							"Buddy Joined %o:\n%a"},

	{"chat_buddy_left",		"Buddy left chatroom",
							"#d2d2d2",	5000,	70,
							GAIMOSD_FLAGS_SHOW_DEFAULT,
							"Buddy Left %o:\n%a"},

	{"chat_invited",		"Invited to chatroom",
							"#7bfc00",	5000,	30,
							GAIMOSD_FLAGS_SHOW_DEFAULT,
							"Chat Invitation to %o\n%a - %m"},

	{"chat_joined",			"Joined chatroom",
							"#ffff00",	3000,	50,
							GAIMOSD_FLAGS_SHOW_DEFAULT,
							"Joined %o\nTopic: %m"},

	{"warning",				"Warned",
							"#ff0000",	3000,	50,
							GAIMOSD_FLAGS_SHOW_ALWAYS,
							"WARNING!\n%a warned you to %o%%!"}
};


#endif /* __GAIMOSD_H__ */
